
// Set known Cal factors for each board
// Curently these are done manually

/* CAL Factor Storage in EEPROM
Use SCPI CAL:VOLT  INDEX, VALUE
INDEX is Cal factor, a 4 byte float
Value is the cal value
Offsets shold be in the mV range, Gains should be 1.0 +/- 5%

1.5V range CAL:
  0 is VF_GAIN, 1 is VF_OFFSET
  2 is VM_GAIN, 3 is VM_OFFSET
15V range CAL:
  4 is VF_GAIN, 5 is VF_OFFSET
  6 is VM_GAIN, 7 is VM_OFFSET
............

Currents start

*/

#define CAL       // Turn off to disable cal. Or use SCPI CALibrate:STATe OFF

void setCal(byte calOn){
int i;
  // Set default (no) calibration for all ranges
  for(i = 0; i < NUM_RANGES; i++){
    Range[i].FOffsCal = 0.0;
    Range[i].FGainCal = 1.0;
    Range[i].MOffsCal = 0.0;
    Range[i].MGainCal = 1.0;
  }
  if (calOn == 1){
  //Serial.println("Calibration Values ON");

#ifdef CAL

// Cal factors are from linear regression of the error, as measured by the DMM.
// For VF, offset cal is the negative of the offset term. Gain is 1- Gain term
// For VM, offset cal is the offset term. Gain is 1- Gain term

//Cal factors for Board #1

// Calibrated at 2021-09-17 11:06:22.578853
// 1.5 V Range Cal
Range[VRange_1_5].FGainCal =  0.98599201 ;
Range[VRange_1_5].FOffsCal =  0.00000794 ;
Range[VRange_1_5].MGainCal =  1.00229606 ;
Range[VRange_1_5].MOffsCal = -0.00187238 ;
// 15 V Range Cal
Range[VRange_15 ].FGainCal =  0.99274970 ;
Range[VRange_15 ].FOffsCal = -0.00155500 ;
Range[VRange_15 ].MGainCal =  1.00230964 ;
Range[VRange_15 ].MOffsCal = -0.00187619 ;
// 150 V Range Cal
Range[VRange_150].FGainCal =  1.01223362 ;
Range[VRange_150].FOffsCal = -0.01591642 ;
Range[VRange_150].MGainCal =  0.98301726 ;
Range[VRange_150].MOffsCal = -0.01869274 ;


// Calibrated 2021-09-17 09:33:28.981669 Temp.: 23.0
// 1e-06 I Range Cal
Range[IRange_1uA].FGainCal =  0.95153783 ;
Range[IRange_1uA].FOffsCal = -0.00000000 ;
Range[IRange_1uA].MGainCal =  1.01904073 ;
Range[IRange_1uA].MOffsCal =  0.00000000 ;
// Calibrated 2021-09-17 09:23:44.182840 Temp.: 23.0
// 1e-05 I Range Cal
Range[IRange_10uA].FGainCal =  0.96175411 ;
Range[IRange_10uA].FOffsCal = -0.00000000 ;
Range[IRange_10uA].MGainCal =  1.00821397 ;
Range[IRange_10uA].MOffsCal = -0.00000000 ;
// Calibrated 2021-09-17 09:23:44.182840 Temp.: 23.0
// 1e-05 I Range Cal
Range[IRange_10uA].FGainCal =  0.96175411 ;
Range[IRange_10uA].FOffsCal = -0.00000000 ;
Range[IRange_10uA].MGainCal =  1.00821397 ;
Range[IRange_10uA].MOffsCal = -0.00000000 ;
// Calibrated 2021-09-17 09:13:31.291638 Temp.: 23.0
// 0.0001 I Range Cal
Range[IRange_100uA].FGainCal =  0.96241581 ;
Range[IRange_100uA].FOffsCal = -0.00000001 ;
Range[IRange_100uA].MGainCal =  1.00752497 ;
Range[IRange_100uA].MOffsCal = -0.00000003 ;
// Calibrated 2021-09-17 09:10:30.862709 Temp.: 23.0
// 0.001 I Range Cal
Range[IRange_1mA].FGainCal =  0.98865149 ;
Range[IRange_1mA].FOffsCal = -0.00000011 ;
Range[IRange_1mA].MGainCal =  0.98077891 ;
Range[IRange_1mA].MOffsCal = -0.00000036 ;
// Calibrated 2021-09-17 08:58:49.774473 Temp.: 23.0
// 0.01 I Range Cal
Range[IRange_10mA].FGainCal =  0.95377510 ;
Range[IRange_10mA].FOffsCal = -0.00000097 ;
Range[IRange_10mA].MGainCal =  1.01664678 ;
Range[IRange_10mA].MOffsCal = -0.00000384 ;
// Calibrated 2021-09-17 09:09:00.902185 Temp.: 23.0
// 0.1 I Range Cal
Range[IRange_100mA].FGainCal =  0.95180155 ;
Range[IRange_100mA].FOffsCal = -0.00001034 ;
Range[IRange_100mA].MGainCal =  1.01875133 ;
Range[IRange_100mA].MOffsCal = -0.00003585 ;

#endif // CAL
  }
  else {
    //Serial.println("Calibration Values OFF");
  }
}
